import math
from datetime import datetime

from sobeit.game import CVector, CColPoint, Game
from sobeit.imgui import ImGui, ImFloat, ImColor
from sobeit.render_api import D3DXVECTOR3

from modsa_lib.core import Chat
from modsa_lib.core.command import Command, CmdArgParser, CmdArgPlayerid
from modsa_lib.imgui.flags import ImGuiColorEditFlags_NoInputs
from modsa_lib.utils.config import ConfigMixin
from modsa_lib.consts.raknet import ID_BULLET_SYNC, ID_AIM_SYNC
from modsa_lib.consts.sobeit import Bone
from modsa_lib.core.handlers import RenderHandler, PacketHandler, HandlersManagementMixin
from modsa_lib.imgui.utils import ImGuiUtils
from modsa_lib.core.bitstream import parse_packet
from modsa_lib.core.sobeit_api import RenderAPI, PlayerAPI
from modsa_lib.utils.common import ColorUtils


class Tracer:
    def __init__(self, origin_pos, target_pos, color: list, time: [float, int], fade_out: bool):
        self.frames = time * 100
        self.frames_all = self.frames
        self.origin_pos = origin_pos
        self.target_pos = target_pos
        self.__fade_out = fade_out
        self.__color = [int(x * 255.0) for x in color]

    def render(self):
        if self.frames > 0:
            self.frames -= 1
            alpha = int(self.frames / self.frames_all * 255) if self.__fade_out else 255
            RenderAPI.draw_line(
                *self.origin_pos,
                *self.target_pos,
                ColorUtils.argb_to_d3d9color(alpha, self.__color[0], self.__color[1], self.__color[2]),
            )
            RenderAPI.draw_sphere(
                *self.target_pos,
                0.02,
                ColorUtils.argb_to_d3d9color(alpha, self.__color[0], self.__color[1], self.__color[2]),
            )


class BaseTracerManager(ConfigMixin, HandlersManagementMixin):
    def __init__(self):
        self.tracers = []
        self.__players = set()

        self.load_config()

        self.handlers = [
            RenderHandler(self.render),
        ]

    def render(self):
        for tracer in self.tracers:
            if tracer.frames <= 0:
                self.tracers.remove(tracer)
                continue

            tracer.render()

    def watch(self, player_id):
        self.__players.add(player_id)

    def unwatch(self, player_id):
        if player_id in self.__players:
            self.__players.remove(player_id)

    def is_watch(self, player_id):
        return player_id in self.__players

    def create_tracer(
        self,
        start_pos: list,
        end_pos: list,
        color: ImColor,
        time: [float, int],
        fade_out: bool = True,
    ):
        color_rgb = [color.Value.x, color.Value.y, color.Value.z]
        self.tracers.append(Tracer(start_pos, end_pos, color_rgb, time, fade_out))


class TracerManager(BaseTracerManager):
    config_path = 'modsa_basic/tracer.json'

    def __init__(self):
        super().__init__()
        self.imgui_colors = {
            'hit': ColorUtils.hex_to_imcolor(self.config['color_hit']),
            'miss': ColorUtils.hex_to_imcolor(self.config['color_miss']),
            'cloud': ColorUtils.hex_to_imcolor(self.config['color_cloud']),
        }

        self.add_handlers(PacketHandler(ID_BULLET_SYNC, self.__handle_packet))
        self.start()

    def get_default_config(self):
        return {
            'color_hit': 'FF0000',
            'color_miss': '0000FF',
            'color_cloud': '00FF00',
            'tracer_time': 5,
        }

    def __handle_packet(self, bs_data, packet_id):
        data = parse_packet(bs_data, packet_id)
        if not self.is_watch(data['id']):
            return
        if sum(data['center']) != 0.0:
            color = self.imgui_colors.get('hit' if data['type'] == 1 else 'miss')
        else:
            color = self.imgui_colors.get('cloud')
            data['target'] = [x * 0.995 for x in data['origin']]

        self.create_tracer(data['origin'], data['target'], color, self.config['tracer_time'])

    def render_traces_config(self, rendering: bool):
        if not rendering:
            self.save_config()
            return

        ImGuiUtils.header(" --- [Трейсеры] ---")
        tracer_colors = self.imgui_colors
        if ImGui.ColorEdit3("Попадание", tracer_colors['hit'], ImGuiColorEditFlags_NoInputs):
            self.config['color_hit'] = ColorUtils.imcolor_to_hex(tracer_colors['hit'])
        if ImGui.ColorEdit3("Промах", tracer_colors['miss'], ImGuiColorEditFlags_NoInputs):
            self.config['color_miss'] = ColorUtils.imcolor_to_hex(tracer_colors['miss'])
        if ImGui.ColorEdit3("Промах (Небо)", tracer_colors['cloud'], ImGuiColorEditFlags_NoInputs):
            self.config['color_cloud'] = ColorUtils.imcolor_to_hex(tracer_colors['cloud'])

        tracer_time = ImFloat(self.config['tracer_time'])
        if ImGui.SliderFloat(f"Время жизни трейсера (сек)", tracer_time, 0.5, 15.0):
            self.config['tracer_time'] = tracer_time.v


class TracerAimManager(BaseTracerManager):
    config_path = 'modsa_basic/aim_tracer.json'

    def __init__(self):
        super().__init__()
        self.__player_watched = None
        self.__last_cam_pos = None
        self.__previous_aim = None
        self.__last_update_datetime = None
        self.__aim_targets = []
        self.imgui_color = ColorUtils.hex_to_imcolor(self.config['aim_color'])

        self.add_handlers(PacketHandler(ID_AIM_SYNC, self.__handle_packet))
        self.create_tracer([0, 0, 0], [0, 0, 0], self.imgui_color, 0, fade_out=False)
        self.start()

    def get_default_config(self):
        return {
            'aim_color': 'dc17ff',
        }

    def __handle_packet(self, bs_data, packet_id):
        data = parse_packet(bs_data, packet_id)
        if not self.is_watch(data['id']):
            return

        time_now = datetime.now()
        frames = 25
        if self.__last_update_datetime:
            time_diff = (time_now - self.__last_update_datetime).total_seconds()
            if time_diff < 0.5:
                frames = round(time_diff * frames)
            self.__last_update_datetime = time_now
        else:
            self.__last_update_datetime = time_now

        if frames == 0:
            return

        self.__last_cam_pos = CVector(*data['cam_pos'])
        cam_front = CVector(*data['cam_front'])

        start_pos = self.get_tracer_start()
        temp_vec = self.__last_cam_pos + cam_front * 300.0
        vec_end = self.__recalc_tracer_end(start_pos, temp_vec)
        if data['cam_mode'] == 53:
            vec_end = self.__add_weapon_offset(self.__last_cam_pos, vec_end, cam_front)
            vec_end = self.__recalc_tracer_end(start_pos, vec_end)

        if not self.__previous_aim:
            self.__aim_targets.append(vec_end.to_list())
            self.__previous_aim = vec_end
            return

        if len(self.__aim_targets) < 300:
            zipped = zip(
                [vec_end.x, vec_end.y, vec_end.z], [self.__previous_aim.x, self.__previous_aim.y, self.__previous_aim.z]
            )
            x_step, y_step, z_step = [(x - y) / frames for (x, y) in zipped]
            for _ in range(0, frames):
                self.__previous_aim += CVector(x_step, y_step, z_step)
                self.__aim_targets.append(self.__previous_aim.to_list())

    @staticmethod
    def __recalc_tracer_end(vec_start: CVector, vec_end: CVector) -> CVector:
        collision = CColPoint()
        if Game.process_line_of_sight(vec_start, vec_end, collision):
            new_pos = collision.get_position()
            return CVector(new_pos.x, new_pos.y, new_pos.z)
        return vec_end

    def __add_weapon_offset(self, vec_start: CVector, vec_end: CVector, cam_front: CVector) -> CVector:
        weapon = PlayerAPI.get_weapon(self.__player_watched)
        if weapon < 21 or weapon > 39:
            return vec_end

        if 21 < weapon < 30 or weapon in [32, 38]:
            offset_z = 22.5
            alpha = -0.045
        elif weapon in [30, 31]:
            vec_diff: CVector = vec_end - vec_start
            vec_diff *= vec_diff
            vec_length = math.sqrt(vec_diff.x + vec_diff.y + vec_diff.z)
            offset_z = 15.5
            alpha = -0.039 + (vec_length * 0.0001)
        elif weapon == 33:
            offset_z = 11
            alpha = -0.025
        else:
            return vec_end

        sin_a = math.sin(alpha)
        cos_a = math.cos(alpha)
        vec_rotated = CVector(
            cam_front.x * cos_a - cam_front.y * sin_a, cam_front.x * sin_a + cam_front.y * cos_a, cam_front.z
        )

        return vec_start + CVector(0, 0, offset_z) + vec_rotated * 300

    def get_tracer_start(self) -> CVector:
        player_pos = PlayerAPI.get_position(self.__player_watched)
        return (
            self.__last_cam_pos
            if RenderAPI.calc_screen_coors(D3DXVECTOR3(*player_pos)).z < 1.0
            else CVector(*PlayerAPI.get_bone_position(Bone.HEAD1, self.__player_watched))
        )

    def render(self):
        if not self.__aim_targets or not PlayerAPI.is_near(self.__player_watched):
            return

        if len(self.__aim_targets) > 1:
            aim_end = self.__aim_targets.pop(0)
        else:
            aim_end = self.__aim_targets[0]

        tracer = self.tracers[0]
        tracer.origin_pos = self.get_tracer_start().to_list()
        tracer.target_pos = aim_end

        tracer.frames = 1
        tracer.render()

    def watch(self, player_id):
        self.__player_watched = player_id
        self.reset_tracer()

    def unwatch(self, player_id=-1):
        self.__player_watched = None
        self.reset_tracer()

    def is_watch(self, player_id):
        return self.__player_watched == player_id

    def reset_tracer(self):
        self.tracers[0].origin_pos = [0, 0, 0]
        self.tracers[0].target_pos = [0, 0, 0]

    def render_traces_config(self, rendering: bool):
        if not rendering:
            self.save_config()
            return

        if ImGui.ColorEdit3("AIM игрока", self.imgui_color, ImGuiColorEditFlags_NoInputs):
            self.config['aim_color'] = ColorUtils.imcolor_to_hex(self.imgui_color)


class TracerCommand:
    def __init__(self):
        self.tr_cmd = Command(
            'tr',
            self.tr_callback,
            arg_parser=CmdArgParser({'player': CmdArgPlayerid()}),
        )

        self.tr_aim_cmd = Command(
            'tr_aim',
            self.tr_aim_callback,
            arg_parser=CmdArgParser({'player': CmdArgPlayerid()}),
        )

    def tr_callback(self, args, cmd, parser_data):
        playerid = parser_data['player']
        name = PlayerAPI.get_name(playerid)
        if tracer_manager.is_watch(playerid):
            Chat.print_text(f'Трейсеры для игрока {name} выключены')
            tracer_manager.unwatch(playerid)
        else:
            Chat.print_text(f'Трейсеры для игрока {name} включены')
            tracer_manager.watch(playerid)

    def tr_aim_callback(self, args, cmd, parser_data):
        playerid = parser_data['player']
        name = PlayerAPI.get_name(playerid)
        if tracer_aim_manager.is_watch(playerid):
            Chat.print_text('AIM трейсеры выключены')
            tracer_aim_manager.unwatch(playerid)
        else:
            Chat.print_text(f'AIM трейсеры переключены на {name}')
            tracer_aim_manager.watch(playerid)


tracer_manager = TracerManager()
tracer_aim_manager = TracerAimManager()
tracer_command = TracerCommand()
