from copy import deepcopy

from sobeit.imgui import ImGui, ImColor
from typing import Callable, Any, Optional

from modsa_lib.core import Chat
from modsa_lib.imgui.fonts import IMGUI_DEFAULT_FONT_20
from modsa_lib.core.handlers import CommandHandler, HandlersManagementMixin
from modsa_lib.core.sobeit_api import PlayerAPI
from modsa_lib.utils.common import bytes_to_text_wrapper


class MinNumberValue:
    def __init__(self, min_value):
        self.min_value = min_value

    def validate(self, value):
        if isinstance(value, int) or isinstance(value, float):
            return value >= self.min_value
        return False

    def print_error(self, arg_name: str = ''):
        Chat.print_text(f'Минимальное значение{" " + arg_name if arg_name else ""}: {self.min_value}')


class MaxNumberValue:
    def __init__(self, max_value):
        self.max_value = max_value

    def validate(self, value):
        if isinstance(value, int) or isinstance(value, float):
            return value <= self.max_value
        return False

    def print_error(self, arg_name: str = ''):
        Chat.print_text(f'Максимальное значение{" " + arg_name if arg_name else ""}: {self.max_value}')


class BaseCommandArg:
    """Базовый класс для обработчиков аргументов"""

    def __init__(
        self,
        required=True,
        default=None,
        validators: list = None,
        cast_type: Callable[..., Any] = str,
        arg_format_name: str = '',
        **kwargs,
    ):
        self.required = required
        self.default = default
        self.validators = validators
        self.cast_type = cast_type
        self.arg_format_name = arg_format_name
        self.kwargs = kwargs

    def clean(self, value):
        """Обработка аргумента. Приведение к типу cast_type"""
        try:
            return self.cast_type(value)
        except (ValueError, TypeError):
            pass

    def get_default(self):
        return self.default

    def get_error_message(self, value):
        return f'Неверный аргумент: {value}'

    def get_arg_format(self, arg_name: str = 'arg'):
        """Получение формата аргумента (Для подсказок)"""
        name = self.arg_format_name
        if not name:
            name = arg_name
        return f'<{name}>' if self.required else f'[{name}]'


class CmdArg(BaseCommandArg):
    """Строковой аргумент команды"""

    def __init__(self, all_text=False, **kwargs):
        super().__init__(**kwargs)
        self.all_text = all_text  # Если True, считываются все оставшиеся аргументы команды


class CmdArgPlayerid(BaseCommandArg):
    """Аргумент ИДа игрока"""

    def __init__(self, **kwargs):
        super().__init__(cast_type=int, **kwargs)

    def clean(self, value):
        playerid = super().clean(value)
        return playerid if playerid is not None and PlayerAPI.is_valid(playerid) else None

    def get_error_message(self, value):
        return f'Игрок с ИДом {value} не найден'


class CmdArgPlayeridName(CmdArg):
    """Аргумент ника / ида игрока"""

    def __init__(self, **kwargs):
        super().__init__(cast_type=int, arg_format_name="ИД или ник", **kwargs)

    def clean(self, value):
        player = super().clean(value)
        return PlayerAPI.get_name(player) if player is not None else value

    def get_error_message(self, value):
        return f'Игрок с ID {value} не найден'


class CmdArgInt(CmdArg):
    """Аргумент целого числа"""

    def __init__(self, **kwargs):
        super().__init__(cast_type=int, **kwargs)

    def get_error_message(self, value):
        return f'Неверное число: {value}'


class CmdArgFloat(CmdArg):
    """Аргумент дробного числа"""

    def __init__(self, **kwargs):
        super().__init__(cast_type=float, **kwargs)

    def get_error_message(self, value):
        return f'Неверное число: {value}'


class CmdArgParser:
    """
    Класс парсера аргументов.
    Обрабатывает аргументы команды в соответствии с шаблоном в виде словаря.
    Имя ключа может быть любым (По ним в дальнейшем будет получение обработанных данных).
    Доступные типы-классы:
        - CommandArg: строка
        - CmdArgPlayerid: ИД игрока. Делает проверку игрока онлайн
        - CommandArgInt: целое число
        - CommandArgFloat - дробное число
    Пример:
    {
        "target": CmdArgPlayerid(),
        "reason": CommandArg(all_text=True, required=False, default="Текст")
    }

    Класс CommandArg принимаем флаг all_text=True, который указыват что необходимо считать все
    оставшиеся аргументы команды.

    Результаты парсера будут в словаре data.
    Если не удалось спарсить значение, его ключ будет удалён из результата
    В результате также будет ключ '_used_default' со списком ключей, которым
    было присвоено значение по умолчанию
    """

    def __init__(self, template: dict):
        self.value_types = deepcopy(template)
        self.__clean()

    def parse(self, cmd_args, cmd) -> bool:
        self.__clean()
        items: list = cmd_args.split(' ')
        items_count = len(items)
        need_args = any([x.required for x in self.value_types.values()])

        if not items[0] and need_args:
            message = f'Используйте: /{cmd} {self.build_args_format()}'
            self.print_error(message)
            return False

        for index, (key, value_parser) in enumerate(self.value_types.items()):
            all_text = hasattr(value_parser, 'all_text') and value_parser.all_text
            value = '' if index > items_count - 1 else ' '.join(items[index:items_count]) if all_text else items[index]
            parse_result = value_parser.clean(value)

            if parse_result is not None:
                self.data[key] = parse_result
                if value_parser.validators:
                    for validator in value_parser.validators:
                        if not validator.validate(self.data[key]):
                            validator.print_error(key)
                            return False
            elif value_parser.required:
                self.print_error(value_parser.get_error_message(value))
                return False
            elif value_parser.default is not None:
                self.data[key] = value_parser.default
                self.data['_used_default'].append(key)

            if all_text:
                break

        return True

    def print_error(self, message):
        Chat.print_text(message)

    def build_args_format(self) -> str:
        return ' '.join([v.get_arg_format(k) for (k, v) in self.value_types.items()])

    def __clean(self):
        self.data = {'_used_default': []}


class Command(HandlersManagementMixin):
    """Класс команды"""

    def __init__(
        self,
        name: str,
        callback: Callable[..., Any],
        pre_callback: Callable[..., Any] = None,
        alias: Optional[list] = None,
        description: str = '',
        arg_parser: CmdArgParser = None,
        auto_start=True,
        **kwargs,
    ):
        """
        name:
        """
        self.name = name
        self.callback = callback
        self.pre_callback = pre_callback
        self.alias = alias
        self.description = description
        self.arg_parser = arg_parser
        self.kwargs = kwargs

        handlers = [CommandHandler(name, self.__command_callback)]
        if alias:
            for alias_name in alias:
                if alias_name == name:
                    continue
                handlers.append(CommandHandler(alias_name, self.__command_callback))
        self.handlers = handlers

        if auto_start:
            self.start()

    @bytes_to_text_wrapper
    def __command_callback(self, args: str, cmd: str):
        if self.pre_callback and not self.pre_callback():
            return

        if self.arg_parser:
            if self.arg_parser.parse(args, cmd):
                self.callback(args=args, cmd=cmd, parser_data=deepcopy(self.arg_parser.data))
        else:
            self.callback(args=args, cmd=cmd)


class CommandDocMixin:
    """
    Миксин для создания документации команд, находящихся в классе
    """

    _commands_info = None

    def build_commands_info(self) -> list:
        if self._commands_info is not None:
            return self._commands_info

        self._commands_info = []
        attrs = vars(self)
        for value in attrs.values():
            if isinstance(value, Command):
                self._commands_info.append(
                    {
                        "name": value.name,
                        "alias": value.alias,
                        "description": value.description,
                        "args_format": value.arg_parser.build_args_format() if value.arg_parser else '',
                    }
                )
        return self._commands_info

    def render_commands_doc(self):
        commands_data = self.build_commands_info()

        ImGui.PushFont(IMGUI_DEFAULT_FONT_20)
        for command in commands_data:
            ImGui.TextColored(ImColor(60, 180, 255), f'/{command["name"]} {command["args_format"]}')
            if command["description"]:
                ImGui.TextWrapped(f'\t{command["description"]}')
            if command["alias"]:
                ImGui.Text(f'\tПсевдонимы: {", ".join(command["alias"])}')
        ImGui.PopFont()
