from sobeit import event_api

from modsa_lib.consts.sobeit import KEYCOMBO_DOWN_PRESSED
from modsa_lib.core.sobeit_api import CommandAPI, RenderAPI


class HandlersManagementMixin:
    """Базовый миксин запуска/остановки обработчиков"""

    __running = False
    __handlers = None

    def after_handlers_start(self):
        """ Дейсвтия при успешном запуске обработчиков """
        pass

    def after_handlers_stop(self):
        """ Дейсвтия при успешной остановке обработчиков """
        pass

    def is_running(self) -> bool:
        return self.__running

    def _set_running(self, state: bool):
        self.__running = state

    @property
    def handlers(self):
        if self.__handlers is None:
            self.__handlers = []
        return self.__handlers

    @handlers.setter
    def handlers(self, value: list):
        if self.is_running():
            raise Exception("Нельзя заменять запущенные обработчики")
        self.__handlers = value

    def add_handlers(self, *args):
        """Добавление новых обработчиков"""
        for handler in args:
            self.handlers.append(handler)
            if self.is_running():
                handler.start()

    def start(self) -> bool:
        """Запуск обработчиков"""
        if self.is_running():
            return False

        self._set_running(True)
        for handler in self.handlers:
            handler.start()
        self.after_handlers_start()
        return True

    def stop(self) -> bool:
        """Остановка обработчиков"""
        if not self.is_running():
            return False

        self._set_running(False)
        for handler in self.handlers:
            handler.stop()
        self.after_handlers_stop()
        return True


class BaseHandler:
    """
    Базовый класс создания обработчиков
    """

    def __init__(self, register_func, remove_func):
        self.__register_func = register_func
        self.__remove_func = remove_func
        self.__event_handler = None
        self.register_args = tuple()

    def is_running(self):
        return self.__event_handler is not None

    def start(self):
        if self.__event_handler is not None:
            return

        self.__event_handler = self.__register_func(*self.register_args)

    def stop(self):
        if self.__event_handler is None:
            return

        self.__remove_func(self.__event_handler)
        self.__event_handler = None


class RPCHandler(BaseHandler):
    """
    Класс создания обработчика RPC пакетов игры
    """

    def __init__(self, event: int, callback):
        """
        event - ID пакета
        callback - вызываемая функция
        """
        super().__init__(event_api.register_rpc, event_api.remove)
        self.register_args = (event, callback)


class PacketHandler(BaseHandler):
    """
    Класс создания обработчика пакетов игры
    """

    def __init__(self, event: int, callback, incoming=True):
        """
        event - ID пакета
        callback - вызываемая функция
        incoming - обработка входящих пакетов, если True, иначе исходящих
        """
        super().__init__(event_api.register_packet, event_api.remove)
        self.register_args = (event, callback, incoming)


class CommandHandler(BaseHandler):
    """
    Класс создания обработчика команд
    """

    def __init__(self, cmd: str, callback):
        """
        cmd - название команды
        callback - вызываемая функция
        """
        super().__init__(CommandAPI.register, CommandAPI.remove)
        self.register_args = (cmd, callback)


class KeycomboHandler(BaseHandler):
    """
    Класс создания обработчика нажатия клавиш
    """

    def __init__(self, keycombo: str, callback, press_type: int = KEYCOMBO_DOWN_PRESSED, send_to_game: bool = False):
        """
        keycombo - клавиша(и)
        callback - вызываемая функция
        send_to_game - отправлять нажатые клавиши игре
        press_type - тип нажатия
        """
        super().__init__(CommandAPI.bind, CommandAPI.unbind)
        self.register_args = (keycombo, callback, press_type, send_to_game)


class RenderHandler(BaseHandler):
    """
    Класс создания обработчика рендера
    """

    def __init__(self, callback):
        """
        callback - вызываемая функция
        """
        super().__init__(RenderAPI.register, RenderAPI.remove)
        self.register_args = (callback,)
