import time
import threading

from modsa_lib.consts.raknet import *
from modsa_lib.core.bitstream import parse_rpc
from modsa_lib.core import Chat, RPCHandler, HandlersManagementMixin
from modsa_lib.states import state_variables


class ChatQueue(HandlersManagementMixin):
    """
    Класс очереди команд / сообщений.
    Выполняет очередь из queue, с задержкой в delay секунд
    При вводе в чат чего-либо во время работы, помещает введенный текст в custom_queue
    custom_queue обрабатывается первее, чем queue
    """

    def __init__(self):
        self.queue = []
        self.custom_queue = []
        self.handlers = [
            RPCHandler(RPC_ServerCommand, self.__command_input),
            RPCHandler(RPC_Chat, self.__chat_input),
        ]
        self.__callback = None
        self.__end_callback = None
        self.__delay = 1.1

    def add(self, data):
        """ Добавление в очередь """
        self.queue.append(data)

    def can_start(self):
        """Может ли быть запущен (Проверка на другие запущенные очереди)"""
        return not state_variables['CHAT_QUEUE_ACTIVE']

    def clear(self):
        self.queue.clear()
        self.custom_queue.clear()

    def after_handlers_start(self):
        state_variables['CHAT_QUEUE_ACTIVE'] = True
        return True

    def after_handlers_stop(self):
        state_variables['CHAT_QUEUE_ACTIVE'] = False
        return True

    def start(self, callback=None, delay: float = 1.1) -> bool:
        """
        Запуск очереди
        Args:
            callback: обработчик, который будет вызываться во время выполнения команд
            delay: задержка
        Return:
            True - если процесс успешно запущен, иначе False
            Очередь не может быть запущена, если работает другая очередь
        """
        if not self.can_start():
            return False

        start_successful = super().start()
        if not start_successful:
            return False

        self.__callback = callback
        self.__end_callback = None
        self.__delay = delay
        t = threading.Thread(target=self.__process, daemon=True)
        t.start()
        return True

    def stop(self):
        """ Остановка выполнения очереди """
        stop_successful = super().stop()
        if not stop_successful:
            return False

        self.queue = []
        self.custom_queue = []
        self.__callback = None
        self.__end_callback = None
        return True

    def sleep(self):
        time.sleep(self.__delay)

    def __command_input(self, bs_data, event):
        data = parse_rpc(bs_data, event)
        self.custom_queue.append(data["text"])
        return False

    def __chat_input(self, bs_data, event):
        data = parse_rpc(bs_data, event)
        self.custom_queue.append(data["text"])
        return False

    def __process(self):
        time.sleep(0.1)
        while self.is_running():
            item = None
            is_custom = False
            if self.custom_queue:
                item = self.custom_queue.pop(0)
                is_custom = True
            elif self.queue:
                item = self.queue.pop(0)

            if self.__callback and not is_custom:
                result = self.__callback(item)
                if result or result is None:
                    self.sleep()
                continue
            elif item:
                Chat.send_text(item)

            time.sleep(self.__delay)
