import re
import sys
from math import sqrt

from sobeit.imgui import ImColor


def get_game_dir(path: str = "") -> str:
    game_root = sys.executable.lower().replace('gta_sa.exe', '')
    return f'{game_root}{path}'


def get_vector3_distance(v1: tuple, v2: tuple):
    """
    Высчитывает расстояние между трёхмерными точками.
    Принимает два списка вида [x, y, z]. Например: [0.0, 1.5, 2]
    """
    vec_sub = [e1 - e2 for e1, e2 in zip(v1, v2)]
    return sqrt(vec_sub[0] * vec_sub[0] + vec_sub[1] * vec_sub[1] + vec_sub[2] * vec_sub[2])


class ColorUtils:
    """Функции для работы с цветами"""

    @staticmethod
    def d3d9color_to_hex(color, split=True):
        c1 = format((color & 0xFF000000) >> 24, 'x').upper()
        c2 = format((color & 0x00FF0000) >> 16, 'x').upper()
        c3 = format((color & 0x0000FF00) >> 8, 'x').upper()
        c4 = format(color & 0x000000FF, 'x').upper()

        hex_colors = [c1, c2, c3, c4]
        for count, hex_color in enumerate(hex_colors):
            if len(hex_color) == 1:
                hex_colors[count] = '0' + hex_color

        if not split:
            return hex_colors[0] + hex_colors[1] + hex_colors[2] + hex_colors[3]

        return hex_colors

    @staticmethod
    def argb_to_d3d9color(a: int = 255, r: int = 255, g: int = 255, b: int = 255):
        c1 = a << 24
        c2 = r << 16
        c3 = g << 8
        c4 = b

        return c1 | c2 | c3 | c4

    @staticmethod
    def hex_to_d3d9color(color):
        color_list = color
        if isinstance(color, str):
            color_list = re.findall("\w\w", color)

        while len(color_list) < 4:
            color_list.append('FF')

        c1 = int(color_list[0], 16) << 24
        c2 = int(color_list[1], 16) << 16
        c3 = int(color_list[2], 16) << 8
        c4 = int(color_list[3], 16)

        return c1 | c2 | c3 | c4

    @staticmethod
    def hex_to_rgb(color: str):
        colors = re.findall('\w\w', color)
        return [int(x, 16) for x in colors]

    @staticmethod
    def rgb_to_hex(r, g, b):
        return '%02x%02x%02x' % (int(r), int(g), int(b))

    @classmethod
    def hex_to_imcolor(cls, color: str):
        colors = cls.hex_to_rgb(color)
        return ImColor(*colors)

    @classmethod
    def imcolor_to_hex(cls, color: ImColor):
        clr = color.Value
        return ColorUtils.rgb_to_hex(clr.x * 255, clr.y * 255, clr.z * 255)


class TextUtils:
    """Функции для работы с текстом"""

    text_encoding = 'cp1251'

    @classmethod
    def bytes_to_text(cls, text: bytes) -> str:
        return text.decode(cls.text_encoding).encode('utf8').decode('utf8')

    @classmethod
    def text_to_bytes(cls, text: str) -> bytes:
        return text.encode("utf8").decode('utf8').encode(cls.text_encoding)

    @staticmethod
    def safe_cast(val, to_type, default=None):
        try:
            return to_type(val)
        except (ValueError, TypeError):
            return default


def bytes_to_text_wrapper(func):
    def _bytes_to_text_wrapper_fn(*args):
        new_args = list()
        for index, arg in enumerate(args):
            if isinstance(arg, bytes):
                arg = TextUtils.bytes_to_text(arg)
            elif isinstance(arg, dict):
                for key, value in arg.items():
                    if isinstance(value, bytes):
                        arg[key] = TextUtils.bytes_to_text(value)

            new_args.append(arg)
        func(*tuple(new_args))

    return _bytes_to_text_wrapper_fn
