import threading
from os import listdir
from os.path import splitext, isfile, join

from sobeit.imgui import (
    ImVec2,
    ImGuiTextFilter,
    ImStringList,
    ImInt,
    ImBool,
    ImIntVec4,
    ImGui,
)

from modsa_lib.core import Chat
from modsa_lib.utils.chat_queue import ChatQueue
from modsa_lib.consts.raknet import RPC_ClientMessage
from modsa_lib.imgui.fonts import IMGUI_DEFAULT_FONT_20
from modsa_lib.core.handlers import RPCHandler, RenderHandler, HandlersManagementMixin
from modsa_lib.imgui.utils import ImguiWindow, ImGuiUtils
from modsa_lib.core.bitstream import parse_rpc
from modsa_lib.core.sobeit_api import GuiAPI


class BaseObjSet(HandlersManagementMixin):
    """ Базовый класс для /setobj и /givehb """

    objs_path = 'mod_sa/objs'
    slots_max = 299

    def __init__(self):
        super().__init__()
        self.imgui_window = self.init_imgui_window()
        self.chat_queue = ChatQueue()
        self.render_handler = RenderHandler(self.render)
        self.handlers = [RPCHandler(RPC_ClientMessage, self.handle_chat_rpc)]

        self.files = self.get_files_list()
        self.imgui_data = {
            'filter': ImGuiTextFilter(),
            'filter_buffer': '',
            'files': ImStringList(list(self.files.keys())),
            'selected': ImInt(),
            'remove_mode': ImBool(),
            'slot_start_end': ImIntVec4(0, self.slots_max),
        }
        self.calc_obj_counts()

    def init_imgui_window(self):
        return ImguiWindow("Установка объектов", size=ImVec2(350, 370))

    def run(self, *args, **kwargs):
        self.render_handler.start()

    def handle_chat_rpc(self, bs_data, packet_id):
        message = parse_rpc(bs_data, packet_id).get('text')
        if message.startswith(" ID может быть от 1") or message.startswith(' /hbject [playerid] '):
            return False

    def parse_txt(self, filename) -> list:
        return []

    def parse_map(self, filename) -> list:
        return []

    def render_additional_fields(self):
        pass

    def click_start(self, filename):
        pass

    def load_file_objects(self, filename) -> list:
        file_ext = splitext(filename)[1].lower()
        if file_ext == '.txt':
            return self.parse_txt(filename)
        elif file_ext == '.map':
            return self.parse_map(filename)
        else:
            raise Exception(f"Неверный формат файла.")

    def build_chat_queue(self, filename, slot_start, slot_end):
        if not filename:
            for slot in range(slot_start, slot_end + 1):
                self.chat_queue.add({"slot": slot, "obj": None})
        else:
            objects = self.load_file_objects(filename)
            slot = slot_start
            for obj in objects:
                self.chat_queue.add({"slot": slot, "obj": obj})
                if slot == slot_end:
                    break
                slot += 1

    def start(self) -> bool:
        if self.is_running():
            return self.stop()

        if not self.chat_queue.can_start():
            Chat.print_text(f"Запущен другой процесс")
            return False

        return super().start()

    def stop(self) -> bool:
        Chat.print_text(f"Установка объектов завершена")
        self.chat_queue.stop()
        return super().stop()

    def start_set_objects(
        self, filename, slot_start, slot_end, delay: float = 1.1
    ) -> bool:
        if slot_start > slot_end:
            Chat.print_text(f"Неверно заданы слоты")
            return False

        is_running = self.start()
        if is_running:
            self.build_chat_queue(filename, slot_start, slot_end)
            Chat.print_text(
                f"Запущен процесс {'удаления' if not filename else 'установки'} объектов"
            )
            self.chat_queue.add(None)
            self.chat_queue.start(self.chat_queue_callback, delay)

        return is_running

    def chat_queue_callback(self, data):
        pass

    def render(self):
        if not self.imgui_window.is_open():
            GuiAPI.samp_toggle_cursor(False)
            Chat.set_chat_block(False)
            return self.render_handler.stop()

        imgui_filter = self.imgui_data.get('filter')
        GuiAPI.samp_toggle_cursor(True)

        with self.imgui_window:
            if imgui_filter.Draw():
                self.imgui_data['filter_buffer'] = imgui_filter.InputBuf
                new_files_list = []
                for file in self.files:
                    if imgui_filter.InputBuf.lower() in file.lower():
                        new_files_list.append(file)
                self.imgui_data['files'] = ImStringList(new_files_list)
            if ImGui.Button("Обновить"):
                self.update_files_list()
                self.__calc_obj_counts_thread()
            ImGui.ListBox(
                "Файлы", self.imgui_data['selected'], self.imgui_data['files'], 8
            )
            try:
                file_index = self.imgui_data['selected'].v
                filename = self.imgui_data['files'].v[file_index]
            except IndexError:
                filename = None

            ImGui.Text(
                f"Объектов в файле: {self.files.get(filename, 0) if filename else 0}"
            )
            ImGui.Checkbox("Удаление объектов", self.imgui_data['remove_mode'])
            ImGui.SameLine()
            ImGuiUtils.help_text('Если отмечено, будет запущен процесс\nочистки слотов')

            self.render_additional_fields()

            ImGui.Text("Слоты (с / по):")
            ImGui.SameLine()
            ImGui.InputInt2("##slots", self.imgui_data['slot_start_end'])
            ImGui.Dummy(ImVec2(0.0, 5.0))
            ImGui.Spacing()
            ImGui.SameLine(ImGui.GetWindowSize().x / 2 - 40)
            ImGui.PushFont(IMGUI_DEFAULT_FONT_20)
            if ImGui.Button("Запустить"):
                if self.imgui_data['remove_mode'].v:
                    filename = None
                self.click_start(filename)
            ImGui.SameLine()
            if ImGui.Button("Остановить"):
                self.stop()
                self.imgui_window.hide()
            ImGui.PopFont()

    def get_files_list(self) -> dict:
        result = {}
        for filename in listdir(self.objs_path):
            if not isfile(join(self.objs_path, filename)):
                continue
            result[filename] = 0

        return result

    def update_files_list(self):
        for filename in listdir(self.objs_path):
            if not isfile(join(self.objs_path, filename)):
                continue
            if filename in self.files:
                continue
            self.files[filename] = 0

    def calc_obj_counts(self):
        t = threading.Thread(target=self.__calc_obj_counts_thread, daemon=True)
        t.start()

    def __calc_obj_counts_thread(self):
        for filename in self.files.keys():
            if self.files[filename] > 0:
                continue
            file_ext = splitext(filename)[1].lower()
            if file_ext == '.txt':
                self.files[filename] = len(self.parse_txt(filename))
            elif file_ext == '.map':
                self.files[filename] = len(self.parse_map(filename))
