import math
import threading
import time
import re
import requests

from sobeit.imgui import ImStringList, ImGui, ImInt

from modsa_lib.core import Chat
from modsa_lib.core.command import CommandDocMixin, Command, CmdArgPlayerid, CmdArgParser
from modsa_lib.utils.config import ConfigMixin


class CipServices:
    IPAPI = 'ipapi'
    SYPEXGEO = 'sypexgeo'

    CHOICES = [(IPAPI, "ip-api.com"), (SYPEXGEO, "sypexgeo.net (без провайдера)")]

    @classmethod
    def get_service_index(cls, service: str) -> int:
        return list(map(lambda x: x[0], cls.CHOICES)).index(service)


class Cip(ConfigMixin, CommandDocMixin):
    config_path = 'srp_basic/cip.json'

    def __init__(self):
        self.cmd_cip = Command(
            'cip',
            self.callback_cip,
            alias=['acip', '.цип'],
            arg_parser=CmdArgParser(
                {
                    'playerid': CmdArgPlayerid(required=False),
                }
            ),
            description='Информация об IP адресах. При acip вывод информации производится в админ чат. '
            'Без указания ида - выводит информацию об адресах в последней строке чата, где оин есть',
        )
        self.load_config()

        self.imgui_services_list = ImStringList(
            list(map(lambda x: x[1], CipServices.CHOICES))
        )
        self.imgui_service = ImInt(
            CipServices.get_service_index(self.config.data.get('service'))
        )

    def get_default_config(self):
        return {'service': CipServices.IPAPI}

    def callback_cip(self, args, cmd, parser_data):
        playerid = parser_data.get('playerid')
        print_local = True if cmd == 'cip' else False
        t = threading.Thread(
            target=self.check_ip,
            daemon=True,
            args=(
                playerid,
                print_local,
            ),
        )
        t.start()

    def check_ip(self, playerid, print_local):
        if playerid:
            Chat.send_text(f'/getip {playerid}')
            time.sleep(1)

        messages = Chat.get_messages()
        for message in messages:
            message_text = message['text']
            if message_text.find(" Nik [") == -1:
                continue

            nickname = re.findall(" Nik \[(\w+)", message_text)
            register_ip = re.findall('R-IP \[(\S+)\]', message_text)
            last_ip = re.findall(' IP \[(\S+)\]', message_text)

            if not register_ip:
                register_ip = re.findall('Register-IP \[(\S+)\]', message_text)
            if not last_ip:
                last_ip = re.findall('Last-IP \[(\S+)\]', message_text)

            if not register_ip or not last_ip:
                Chat.print_text("Ошибка извлечения IP адресов")
                return

            register_ip = register_ip[0]
            last_ip = last_ip[0]

            Chat.print_text("Загрузка информации. Пожалуйста, подождите...")
            regip_data, lastip_data, dist = self.get_ips_info(register_ip, last_ip)
            if not regip_data or not lastip_data:
                Chat.print_text("Ошибка проверки IP адресов")
                return

            player_name = nickname[0] if nickname else ""
            if print_local:
                Chat.print_text(
                    "{{3DCCFF}}{0}{{FFFF00}} | Страна: {1} | Город: {2}".format(
                        register_ip, regip_data["country"], regip_data["city"]
                    )
                )
                Chat.print_text(
                    f'{{FFFF00}}Провайдер R-IP: {regip_data["provider"]}'
                )
                Chat.print_text(
                    "{{3DCCFF}}{0}{{FFFF00}} | Страна: {1} | Город: {2}".format(
                        last_ip, lastip_data["country"], lastip_data["city"]
                    )
                )
                Chat.print_text(
                    f'{{FFFF00}}Провайдер R-IP: {lastip_data["provider"]}'
                )
                if dist is not None:
                    Chat.print_text(f'{{3DCCFF}}Расстояние между IP: {dist} км')
            else:
                Chat.send_text(
                    f'/a RegIP: {regip_data["country"]} | {regip_data["city"]} | {regip_data["provider"]}'
                )
                time.sleep(1)
                Chat.send_text(
                    f'/a IP: {lastip_data["country"]} | {lastip_data["city"]} | {lastip_data["provider"]}'
                )
                time.sleep(1)
                if dist is not None:
                    time.sleep(1)
                    Chat.send_text(
                        f'/a Расстояние: {dist} км | Игрок: {player_name}'
                    )

            return
        Chat.print_text("Ипы для сравнения не найдены")

    def get_ips_info(self, reg_ip, last_ip):
        ips_equal = reg_ip == last_ip
        dist = 0 if ips_equal else None
        service = self.config.data.get('service')

        service_func = self.service_ipapi
        if service == CipServices.SYPEXGEO:
            service_func = self.service_sypexgeo

        regip_data = service_func(reg_ip)
        time.sleep(1)
        if ips_equal:
            lastip_data = regip_data
        else:
            lastip_data = service_func(last_ip)

        if dist is None and regip_data and lastip_data:
            dist = self.ip_dist(
                regip_data["lat"],
                regip_data["lon"],
                lastip_data["lat"],
                lastip_data["lon"],
            )

        return regip_data, lastip_data, dist

    def render_settings(self, rendering: bool):
        if rendering:
            ImGui.Text("Сервис проверки:")
            if ImGui.Combo("##ipservice", self.imgui_service, self.imgui_services_list):
                self.config.data["service"] = CipServices.CHOICES[self.imgui_service.v][0]
        else:
            self.config.save()

    @staticmethod
    def ip_dist(a1, a2, b1, b2):
        earth_r = 6373.0
        lat1 = math.radians(a1)
        lat2 = math.radians(b1)
        lon1 = math.radians(a2)
        lon2 = math.radians(b2)

        dlon = lon2 - lon1
        dlat = lat2 - lat1

        a = (
            math.sin(dlat / 2) ** 2
            + math.cos(lat1) * math.cos(lat2) * math.sin(dlon / 2) ** 2
        )
        c = 2 * math.atan2(math.sqrt(a), math.sqrt(1 - a))

        return int(earth_r * c)

    @staticmethod
    def service_ipapi(ip):
        r = requests.get(f'http://ip-api.com/json/{ip}')
        if r.status_code != 200:
            return None

        request_data = r.json()
        if request_data["status"] != "success":
            Chat.print_text(f'CIP error: {request_data["message"]}')
            return None

        return {
            'lon': request_data['lon'],
            'lat': request_data['lat'],
            'country': request_data['country'],
            'city': request_data['city'],
            'provider': request_data["org"] or request_data["isp"],
        }

    @staticmethod
    def service_sypexgeo(ip):
        r = requests.get(f'http://api.sypexgeo.net/json/{ip}')
        if r.status_code != 200:
            return None

        request_data = r.json()
        if request_data["error"]:
            Chat.print_text(f'CIP error: {request_data["error"]}')
            return None

        return {
            'lon': request_data["city"]["lon"],
            'lat': request_data["city"]["lat"],
            'country': request_data["country"]["name_ru"],
            'city': request_data["city"]["name_ru"],
            'provider': "-",
        }


cip_instance = Cip()
