import re

from sobeit.imgui import ImGui, ImBool, ImVec2, ImFloat, ImColor, ImVec4

from modsa_lib.core import Chat
from modsa_lib.core.keycombo import KeycomboPressed, KeycomboUpdaterMixin
from modsa_lib.imgui.flags import *
from modsa_lib.utils.config import ConfigMixin
from modsa_lib.consts.raknet import RPC_ClientMessage
from modsa_lib.consts.sobeit import EVENT_GAME_INIT
from modsa_lib.core.event import Event
from modsa_lib.imgui.fonts import IMGUI_DEFAULT_FONT_18, IMGUI_DEFAULT_FONT_19
from modsa_lib.core.handlers import HandlersManagementMixin, RenderHandler, RPCHandler
from modsa_lib.imgui.utils import ImGuiUtils
from modsa_lib.core.bitstream import parse_rpc
from modsa_lib.core.sobeit_api import RenderAPI
from modsa_lib.utils.common import ColorUtils


class ReportMessageType:
    REPORT = 0
    QUESTION = 1
    ANSWER = 2


class ReportRecon(ConfigMixin, KeycomboUpdaterMixin):
    """ Переход в рекон по репорту """

    config_path = 'srp_basic/report_recon.json'

    def __init__(self):
        super().__init__()
        self.__key_update = None

        self.load_config()
        self.set_keycombo_handlers(
            {
                'recon_report_key': KeycomboPressed(
                    self.config.data.get('recon_report_key'), self.handle_report_keys
                ),
                'recon_warning_key': KeycomboPressed(
                    self.config.data.get('recon_warning_key'), self.handle_warning_keys
                ),
            }
        )

        for handler in list(self.keycombo_handlers.values()):
            handler.start()

    def get_default_config(self):
        return {'recon_report_key': 'z', 'recon_warning_key': 'x'}

    def handle_report_keys(self):
        if report_detach.is_running():
            messages = report_detach.reports[-30:]
            messages.reverse()
        else:
            messages = Chat.get_messages()
            messages = [x for x in messages if x["color"] == "D97700"]

        for message in messages:
            if message['text'].find(" Жалоба от") != -1:
                suspect = re.findall(" на \w+\[(\d+)]:", message['text'])
                if suspect:
                    return Chat.send_text(f"/re {suspect[0]}")

    def handle_warning_keys(self):
        messages = Chat.get_messages()
        for message in messages:
            if message['text'].startswith(' <Warning>'):
                suspect = re.findall("\w+\[(\d+)]:", message['text'])
                if suspect:
                    return Chat.send_text(f"/re {suspect[0]}")

    def render_settings(self, rendering: bool):
        report_detach.edit_mode = rendering

        if rendering:
            ImGuiUtils.header('--- [Report] ---')
            ImGui.PushFont(IMGUI_DEFAULT_FONT_19)

            if self.imgui_keycombo("recon_report_key", "Переход в рекон по жалобе"):
                self.config.data["recon_report_key"] = self.get_new_keycombo()

            if self.imgui_keycombo("recon_warning_key", "Переход в рекон по варнингу"):
                self.config.data["recon_warning_key"] = self.get_new_keycombo()

            ImGui.PopFont()
        else:
            self.save_config()


class ReportDetach(ConfigMixin, HandlersManagementMixin):
    config_path = 'srp_basic/report_detach.json'

    def __init__(self):
        self.load_config()
        self.handlers = [
            RenderHandler(self.render),
            RPCHandler(RPC_ClientMessage, self.handle_rpc),
        ]
        self.reports = []
        self.pos = ImVec2()
        self.size = ImVec2()
        self.font_scale = 1.0
        self.edit_mode = False
        self.imgui = {
            'enabled': ImBool(self.config.data['enabled']),
            'font_size': ImFloat(self.config.data['font_size']),
            'color_question': ColorUtils.hex_to_imcolor(
                self.config.data['color_question']
            ),
            'color_report': ColorUtils.hex_to_imcolor(self.config.data['color_report']),
            'color_answer': ColorUtils.hex_to_imcolor(self.config.data['color_answer']),
        }

        if self.config.data['enabled']:
            self.start()

    def get_default_config(self):
        return {
            'enabled': True,
            'font_size': 1.0,
            'pos_x': 0.45,
            'pos_y': 30,
            'size_x': 600,
            'size_y': 220,
            'color_question': 'ffffff',
            'color_report': 'ffffff',
            'color_answer': 'ffffff',
        }

    def handle_rpc(self, bs_data, packet_id):
        message = parse_rpc(bs_data, packet_id)
        if message["color"] != "D97700":
            return

        message_type = ReportMessageType.QUESTION
        if message["text"].startswith(' Жалоба от'):
            message_type = ReportMessageType.REPORT
        elif message["text"].startswith('<-'):
            message_type = ReportMessageType.ANSWER

        self.reports.append(
            {
                "text": message["text"],
                "type": message_type,
            }
        )
        if len(self.reports) == 201:
            self.reports.pop(0)
        return False

    def render(self):
        ImGui.PushStyleVar(ImGuiStyleVar_WindowRounding, 12.0)
        ImGui.PushStyleVar(ImGuiStyleVar_ItemSpacing, ImVec2(0, 0))
        ImGui.PushStyleColor(ImGuiCol_TitleBgActive, ImVec4(ImColor(0, 0, 0)))
        ImGui.PushFont(IMGUI_DEFAULT_FONT_18)

        ImGui.SetNextWindowPos(self.pos, ImGuiCond_FirstUseEver)
        ImGui.SetNextWindowSize(self.size, ImGuiCond_FirstUseEver)
        ImGui.SetNextWindowBgAlpha(0.8)
        ImGui.Begin(
            name=f"Alt+1: вкл/выкл курсор##report_detach",
            flags=ImGuiWindowFlags_HorizontalScrollbar,
        )
        ImGui.SetWindowFontScale(self.font_scale)

        if self.edit_mode:
            pos = ImGui.GetWindowPos()
            size = ImGui.GetWindowSize()
            self.config.data['pos_x'] = pos.x
            self.config.data['pos_y'] = pos.y
            self.config.data['size_x'] = size.x
            self.config.data['size_y'] = size.y

        for report in self.reports:
            message_color = self.imgui['color_question'].Value
            if report["type"] == ReportMessageType.REPORT:
                message_color = self.imgui['color_report'].Value
            elif report["type"] == ReportMessageType.ANSWER:
                message_color = self.imgui['color_answer'].Value

            ImGui.PushStyleColor(ImGuiCol_Text, message_color)
            ImGui.TextUnformatted(report['text'])
            ImGui.PopStyleColor()

        ImGui.SetScrollHereY(1.0)
        ImGui.End()

        ImGui.PopFont()
        ImGui.PopStyleVar(2)
        ImGui.PopStyleColor()

    def render_settings(self, rendering: bool):
        self.edit_mode = rendering

        if rendering:
            ImGui.PushFont(IMGUI_DEFAULT_FONT_19)
            if ImGui.Checkbox("Вынести репорт в отдельное окно", self.imgui['enabled']):
                self.config.data["report_detach"] = self.imgui['enabled'].v
                self.start() if self.imgui['enabled'].v else self.stop()
            ImGui.Text("Размер шрифта отдельного репорта:")
            if ImGui.SliderFloat(
                "##report_detach_font_size", self.imgui['font_size'], 0.3, 5.0
            ):
                self.config.data["report_detach_font_size"] = self.imgui['font_size'].v
                self.font_scale = self.imgui['font_size'].v
            if ImGui.ColorEdit3(
                "Цвет вопросов в окне",
                self.imgui['color_question'],
                ImGuiColorEditFlags_NoInputs,
            ):
                self.config.data['color_question'] = ColorUtils.imcolor_to_hex(
                    self.imgui['color_question']
                )
            if ImGui.ColorEdit3(
                "Цвет жалоб в окне",
                self.imgui['color_report'],
                ImGuiColorEditFlags_NoInputs,
            ):
                self.config.data['color_report'] = ColorUtils.imcolor_to_hex(
                    self.imgui['color_report']
                )

            if ImGui.ColorEdit3(
                "Цвет ответов в окне",
                self.imgui['color_answer'],
                ImGuiColorEditFlags_NoInputs,
            ):
                self.config.data['color_answer'] = ColorUtils.imcolor_to_hex(
                    self.imgui['color_answer']
                )
            ImGui.PopFont()
        else:
            self.save_config()


report_detach = ReportDetach()
report_recon = ReportRecon()


@Event.register(EVENT_GAME_INIT)
def on_game_init():
    report_detach.pos = RenderAPI.parse_pos(
        report_detach.config.data["pos_x"], report_detach.config.data["pos_y"]
    )
    report_detach.size = ImVec2(
        report_detach.config.data["size_x"], report_detach.config.data["size_y"]
    )
