import re
from typing import Optional

from bs4 import BeautifulSoup
from sobeit.imgui import ImGui, ImVec2

from modsa_lib.core import Chat
from modsa_lib.utils.chat_queue import ChatQueue
from modsa_lib.consts.sobeit import DIALOG_LIST
from modsa_lib.core.handlers import HandlersManagementMixin
from modsa_lib.imgui.utils import ImGuiUtils, ImguiWindow
from modsa_lib.core.sobeit_api import PlayerAPI, GuiAPI
from modsa_lib.utils.common import bytes_to_text_wrapper
from srp_basic.bases import BaseObjSet
from srp_basic.const import SRP_ORGANIZATIONS

chat_queue = ChatQueue()


class TaskGarm(HandlersManagementMixin):
    def __init__(self):
        self.armour_amount = 100

    def start(self) -> bool:
        if self.is_running():
            return False

        if not chat_queue.can_start():
            Chat.print_text("[garm] Запущен другой процесс")
            return False

        return super().start()

    def after_handlers_start(self):
        for player in PlayerAPI.get_players(stream=True):
            chat_queue.add(player)

        Chat.print_text("[garm] Запущена выдача брони. Остановка: /garm")
        chat_queue.start(callback=self.__queue_callback)

    def after_handlers_stop(self):
        Chat.print_text("[garm] Выдача брони завершена")
        chat_queue.stop()

    def __queue_callback(self, item):
        if item:
            Chat.send_text(f"/setarm {item['id']} {self.armour_amount}")
            Chat.print_text(f"Выдана броня: {item['name']} [{item['id']}]", "5096E6")
        else:
            self.stop()


class TaskHHP(HandlersManagementMixin):
    def __init__(self):
        self.hp_amount = 100

    def start(self) -> bool:
        if self.is_running():
            return False

        if not chat_queue.can_start():
            Chat.print_text("[hhp] Запущен другой процесс")
            return False

        return super().start()

    def after_handlers_start(self):
        players = PlayerAPI.get_players(stream=True)
        for player in players:
            chat_queue.add(f'/sethp {player["id"]} {self.hp_amount}')

        Chat.print_text("[hhp] Запущен процесс выдачи HP")
        chat_queue.start(callback=self.__queue_callback)

    def after_handlers_stop(self):
        Chat.print_text("[hhp] Процесс выдачи HP завершён")
        chat_queue.stop()

    def __queue_callback(self, item):
        Chat.send_text(item) if item else self.stop()


class TaskML:
    def __init__(self):
        self.playerid = None

    def show_modal(self, playerid: int):
        self.playerid = playerid
        fractions = ["[0] Снять с лидерки"]

        for frac_id, frac_name in SRP_ORGANIZATIONS.items():
            fractions.append(f"[{frac_id}] {frac_name}")

        GuiAPI.show_samp_dialog(
            callback=self.dialog_callback,
            dialog_type=DIALOG_LIST,
            caption="Выберите лидерку",
            button1="Назначить",
            button2="Отмена",
            text='\n'.join(fractions),
        )

    @bytes_to_text_wrapper
    def dialog_callback(self, data):
        if data['button'] != 1:
            return

        Chat.send_text(f'/makeleader {self.playerid} {data["list_item"]}')


class TaskSetObj(BaseObjSet):
    def run(self):
        super().run()
        Chat.set_chat_block(True)
        self.imgui_window.show()

    def parse_txt(self, filename) -> list:
        objects = []
        with open(f"{self.objs_path}/{filename}", "r") as file:
            for line in file.readlines():
                obj_data = re.findall(r'CreateObject\((.*)\);', line)
                if not obj_data:
                    continue

                try:
                    obj_data = [
                        float(x.replace(' ', '')) for x in obj_data[0].split(',')
                    ]
                except ValueError as e:
                    print(f"Ошибка парсинга в файле: {self.objs_path}/{filename}")
                    print(f"Строка: {line}")
                    break

                objects.append(
                    {
                        "model": int(obj_data[0]),
                        "x": obj_data[1],
                        "y": obj_data[2],
                        "z": obj_data[3],
                        "rx": round(obj_data[4], 4),
                        "ry": round(obj_data[5], 4),
                        "rz": round(obj_data[6], 4),
                    }
                )
        return objects

    def parse_map(self, filename) -> list:
        objects = []

        with open(
            f"{self.objs_path}/{filename}", "r", encoding="utf8", errors='ignore'
        ) as f:
            contents = f.read()
            soup = BeautifulSoup(contents, "lxml")

            items = soup.find_all("object")
            for item in items:
                try:
                    objects.append(
                        {
                            "model": int(item["model"]),
                            "x": float(item['posx']),
                            "y": float(item['posy']),
                            "z": float(item['posz']),
                            "rx": round(float(item.get('rotx', 0.0)), 4),
                            "ry": round(float(item.get('roty', 0.0)), 4),
                            "rz": round(float(item.get('rotz', 0.0)), 4),
                        }
                    )
                except KeyError as e:
                    print(f"Ошибка парсинга в файле: {self.objs_path}/{filename}")
                    print(f"Описание ошибки: {e}")
                    break

        return objects

    def click_start(self, filename):
        slots = self.imgui_data['slot_start_end']
        if self.start_set_objects(filename, slots.v0, slots.v1, delay=0.05):
            self.imgui_window.hide()

    def chat_queue_callback(self, data):
        if not data:
            return self.stop()

        player_pos = PlayerAPI.get_position()
        obj = data.get("obj")
        if not obj:
            return Chat.send_text(f'/obj {data.get("slot")} -1')

        obj_pos = [
            round(obj["x"] - player_pos[0], 4),
            round(obj["y"] - player_pos[1], 4),
            round(obj["z"] - player_pos[2], 4),
        ]

        Chat.send_text(
            f'/obj {data.get("slot")} {obj["model"]} '
            f'{obj_pos[0]} {obj_pos[1]} {obj_pos[2]} '  # pos
            f'{obj["rx"]} {obj["ry"]} {obj["rz"]}'  # rot
        )


class TaskGiveHB(BaseObjSet):
    objs_path = 'mod_sa/hbjects'
    slots_max = 9

    def __init__(self):
        super().__init__()
        self.playerid = None
        self.__prev_player = None

    def init_imgui_window(self):
        return ImguiWindow("Установка объектов", size=ImVec2(350, 400))

    def run(self, playerid: Optional[int], filename: Optional[str]):
        self.playerid = playerid
        if playerid is not None and filename:
            filtered_files = list(filter(lambda x: filename in x, self.files))
            if len(filtered_files) > 0:
                self.start_set_objects(filtered_files[0], 0, self.slots_max)
            else:
                Chat.print_text("Указанный файл не найден")
            return False

        super().run()
        self.imgui_window.show()

    def click_start(self, filename):
        slots = self.imgui_data['slot_start_end']
        if self.start_set_objects(filename, slots.v0, slots.v1):
            self.imgui_window.hide()

    def parse_txt(self, filename) -> list:
        objects = []
        with open(f"{self.objs_path}/{filename}", "r") as file:
            for line in file.readlines():
                if len(line) > 10:
                    objects.append(line.rstrip())
        return objects

    def render_additional_fields(self):
        target = (
            'игрокам в зоне прорисовки'
            if self.playerid is None
            else f'игроку {PlayerAPI.get_name(self.playerid)}'
        )
        ImGui.Text(f"Выдавать: {target}")
        ImGui.SameLine()
        ImGuiUtils.help_text('Для выдачи конкретному игроку используйте /givehb id')

    def build_chat_queue(self, filename, slot_start, slot_end):
        if self.playerid is None:
            players = [x['id'] for x in PlayerAPI.get_players(stream=True)]
        else:
            players = [self.playerid]

        if not filename:
            for player in players:
                for slot in range(slot_start, slot_end + 1):
                    self.chat_queue.add(
                        {'id': player, 'name': PlayerAPI.get_name(player), 'obj': f"{slot} -1"}
                    )
        else:
            objects = self.load_file_objects(filename)
            for player in players:
                slot = slot_start
                for count, obj in enumerate(objects):
                    self.chat_queue.add(
                        {'id': player, 'name': PlayerAPI.get_name(player), 'obj': obj}
                    )
                    if slot == slot_end:
                        break
                    slot += 1

    def chat_queue_callback(self, data):
        if not data:
            self.stop()

        if self.__prev_player is None:
            self.__prev_player = data
        if self.__prev_player.get('id') != data.get('id'):
            Chat.print_text(
                f'Установлены объекты игроку: {self.__prev_player["name"]} [{self.__prev_player["id"]}]'
            )
            self.__prev_player = data

        obj = data.get("obj") or "-1"
        Chat.send_text(f'/hbject {data.get("id")} {obj}')


class TaskOffuninv(HandlersManagementMixin):
    list_path = 'mod_sa/data/offuninvite.txt'

    def start(self) -> bool:
        if self.is_running():
            return False

        if not chat_queue.can_start():
            Chat.print_text("[offuninv] Запущен другой процесс")
            return False

        return super().start()

    def after_handlers_start(self):
        try:
            with open(self.list_path, 'r', encoding='utf8') as file:
                for line in file.readlines():
                    if line and line[0] != '#':
                        chat_queue.add(f'/aoffuninvite {line}')
        except FileNotFoundError:
            Chat.print_text(f"[offuninv] Файл {self.list_path} не найден")
            self.stop()
            return

        Chat.print_text("[offuninv] Запущен процесс массового увольнения")
        chat_queue.start(callback=self.__queue_callback)

    def after_handlers_stop(self):
        Chat.print_text("[offuninv] Процесс массового увольнения завершён")
        chat_queue.stop()

    def __queue_callback(self, item):
        Chat.send_text(item) if item else self.stop()
